--[[ ---------------------------------------------------------------------------

DebuffEnough: Debuff monitor

DebuffEnough is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DebuffEnough is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with DebuffEnough.	If not, see <http://www.gnu.org/licenses/>.

----------------------------------------------------------------------------- ]]

--[[ ---------------------------------------------------------------------------
	 Addon and libraries
----------------------------------------------------------------------------- ]]

local L = LibStub("AceLocale-3.0"):GetLocale("DebuffEnough")
local C = LibStub("AceConfigDialog-3.0")


--[[ ---------------------------------------------------------------------------
	 Create dropdowns
----------------------------------------------------------------------------- ]]

local logLevelsDropdown = {}

if DebuffEnough.logLevels then
	for logname,id in pairs(DebuffEnough.logLevels) do
		logLevelsDropdown[id] = L[logname]
	end
end


--[[ ---------------------------------------------------------------------------
	 Config options
----------------------------------------------------------------------------- ]]
DebuffEnough.options = {
	name = "DebuffEnough",
	handler = DebuffEnough,
	type = "group",
	args = {
		general = {
			type = "group",
			name = L["General"],
			order = 0,
			set = "SetProfileParam",
			get = "GetProfileParam",
			cmdHidden = true,
			args = {
				enable = {
					type = "toggle",
					name = L["Enable"],
					desc = L["Enable or disable the addon"],
					width = "full",
					order = 0,
					set = function(info, v)
						DebuffEnough:SetProfileParam("enable", v)
						if v then
							DebuffEnough:DoEnable()
						else
							DebuffEnough:DoDisable()
						end
				  	end	
				},
				solo = {
					type = "toggle",
					name = L["Enable While Solo"],
					desc = L["Enable DebuffEnough while not in a party/raid"],
					width = "full",
					order = 1,
					set = function(info, v)
							DebuffEnough:SetProfileParam("solo", v)
							DebuffEnough:CheckRaidChange()
				  		end	
				},
				logLevel = {
					type = "select",
					name = L["Log Level"],
					desc = L["Determines the amount of output from the addon"],
					order = 99,
					values = logLevelsDropdown,
					disabled = function() return not LibStub("LibLogger-1.0", true) end,
				},
			},
		},
		display = {
			type = "group",
			name = L["Display"],
			cmdHidden = true,
			order = 1,
			args = {
				show = {
					type = "toggle",
					name = L["Show Display"],
					desc = L["Shows the display"],
					order = 0,
					width = "full",
					get = function(info) return DebuffEnough:GetProfileParam("show") end,
					set = function(info, v)
							DebuffEnough:SetProfileParam("show", v)
							DebuffEnough:UpdateVisible()
				  		end	
				},
				lock = {
					type = "toggle",
					name = L["Lock"],
					desc = L["Lock the display"],
					order = 1,
					width = "full",
					get = function(info) return DebuffEnough:GetProfileParam("lock") end,
					set = function(info, v)
					  		DebuffEnough:SetProfileParam("lock", v)
					  		DebuffEnough:UpdateDisplay()
				  		end	
				},
				scale = {
					type = "range",
					name = L["Scale"],
					desc = L["Scale of the display"],
					min = 1,
					max = 150,
					step = 1,
					bigStep = 1,
					order = 2,
					width = "full",
					get = function(info) return DebuffEnough:GetProfileParam("scale") end,
					set = function(info, v)
							DebuffEnough:SetProfileParam("scale", v)
			  				DebuffEnough:UpdateDisplay()
						end						
				},
				alpha = {
					type = "range",
					name = L["Opacity"],
					desc = L["Opacity of the display"],
					min = 0,
					max = 1,
					step = 0.01,
					bigStep = 0.05,
					order = 3,
					width = "full",
					get = function(info) return DebuffEnough:GetProfileParam("alpha") end,
					set = function(info, v)
				 			DebuffEnough:SetProfileParam("alpha", v)
				  			DebuffEnough:UpdateDisplay()
						end	
				},
				strata = {
					type = "select",
					name = L["Strata"],
					desc = L["Strata of the display"],
					order = 4,
					values = {HIGH = _G["HIGH"], MEDIUM = _G["AUCTION_TIME_LEFT2"], LOW = _G["LOW"], BACKGROUND = _G["BACKGROUND"]},
					get = function(info) return DebuffEnough:GetProfileParam("strata") end,
					set = function(info, v)
				 			DebuffEnough:SetProfileParam("strata", v)
				  			DebuffEnough:UpdateDisplay()
						end	
				},
				fade = {
					type = "range",
					name = L["Fade"],
					desc = L["Fade frame after this many seconds of being debuff enough. Set to 0 for no fading"],
					min = 0,
					max = 60,
					step = 1,
					bigStep = 1,
					width = "full",
					order = 5,
					get = function(info) return DebuffEnough:GetProfileParam("fade") end,
					set = function(info, v)
							DebuffEnough:SetProfileParam("fade", v)
							DebuffEnough:UpdateDisplay()
				  		end	
				},
				debuffcolor = {
					type = "color",
					name = L["Debuff Color"],
					desc = L["Color to display when your target is debuff enough"],
					hasAlpha = false,
					width = "full",
					get = function(info)
							local r = DebuffEnough:GetProfileParam("debuffcolorr")
							local g = DebuffEnough:GetProfileParam("debuffcolorg")
							local b = DebuffEnough:GetProfileParam("debuffcolorb")
							return r, g, b
					end,
					set = function(info, r, g, b)
							DebuffEnough:SetProfileParam("debuffcolorr", r)
							DebuffEnough:SetProfileParam("debuffcolorg", g)
							DebuffEnough:SetProfileParam("debuffcolorb", b) 
							DebuffEnough:UpdateDisplay()
						end
				},
				undebuffcolor = {
					type = "color",
					name = L["Undebuff Color"],
					desc = L["Color to display when your target is not debuff enough"],
					hasAlpha = false,
					width = "full",
					get = function(info)
							local r = DebuffEnough:GetProfileParam("undebuffcolorr")
							local g = DebuffEnough:GetProfileParam("undebuffcolorg")
							local b = DebuffEnough:GetProfileParam("undebuffcolorb")
							return r, g, b
						end,
					set = function(info, r, g, b)
							DebuffEnough:SetProfileParam("undebuffcolorr", r)
							DebuffEnough:SetProfileParam("undebuffcolorg", g)
							DebuffEnough:SetProfileParam("undebuffcolorb", b) 
							DebuffEnough:UpdateDisplay()
						end
				},
				warndebuffcolor = {
					type = "color",
					name = L["Warn Debuff Color"],
					desc = L["Color to display when the optimal debuff for a slot is not being applied"],
					hasAlpha = false,
					width = "full",
					get = function(info)
							local r = DebuffEnough:GetProfileParam("warndebuffcolorr")
							local g = DebuffEnough:GetProfileParam("warndebuffcolorg")
							local b = DebuffEnough:GetProfileParam("warndebuffcolorb")
							return r, g, b
						end,
					set = function(info, r, g, b)
							DebuffEnough:SetProfileParam("warndebuffcolorr", r)
							DebuffEnough:SetProfileParam("warndebuffcolorg", g)
							DebuffEnough:SetProfileParam("warndebuffcolorb", b) 
							DebuffEnough:UpdateDisplay()
						end
				},

			},
		},
		tooltip = {
			type = "group",
			name = L["Tooltip"],
			order = 2,
			set = "SetProfileParam",
			get = "GetProfileParam",
			cmdHidden = true,
			args = {
				showbest = {
					type = "toggle",
					name = L["Show Best Debuff Only"],
					desc = L["Only show the best debuff for the given slot. If unchecked, all possible debuffs are shown, and the best options are highlighted."],
					width = "full",
					order = 2,
					set = function(info, v)
						DebuffEnough:SetProfileParam("showbest", v)
						DebuffEnough:RenderTooltip()
				  	end	,
				},
				showclass= {
					type = "toggle",
					name = L["Show My Class Debuffs Only"],
					desc = L["Only show debuffs that your class can cast"],
					width = "full",
					order = 3,
					set = function(info, v)
						DebuffEnough:SetProfileParam("showclass", v)
						DebuffEnough:GetAvailableDebuffs()
				  	end	,
				},
			},
		},
		custom = {
			type = "group",
			name = L["Custom"],
			order = 3,
			set = "SetProfileParam",
			get = "GetProfileParam",
			cmdHidden = true,
			args = {
				showhealingtaken = {
					type = "toggle",
					name = L["Show Healing Taken Debuffs"],
					desc = L["Show debuffs that reduce healing taken"],
					width = "full",
					order = 2,
					set = function(info, v)
						DebuffEnough:SetProfileParam("showhealingtaken", v)
						DebuffEnough:GetAvailableDebuffs()
				  	end	,
				},
				showcastspeed= {
					type = "toggle",
					name = L["Show Cast Speed Debuffs"],
					desc = L["Show debuffs that reduce cast speed"],
					width = "full",
					order = 3,
					set = function(info, v)
						DebuffEnough:SetProfileParam("showcastspeed", v)
						DebuffEnough:GetAvailableDebuffs()
				  	end	,
				},
			},
		},
	},
}

DebuffEnough.options.args[L["config"]] = {
	type = "execute",
	name = L["config"],
	desc = L["Toggle the Configuration Dialog"],
	func = "ToggleConfigDialog",
	guiHidden = true,
	order = 1,
}


--[[ ---------------------------------------------------------------------------
	 Config defaults
----------------------------------------------------------------------------- ]]
DebuffEnough.defaults = {
	profile = {
		enable = true,
		solo = true,
		logLevel = 0,
		show = true,
		lock = false,
		alpha = 1,
		scale = 100,
		strata = "BACKGROUND",
		fade = 0,
		debuffcolorr = 0,
		debuffcolorg = .75,
		debuffcolorb = 0,
		undebuffcolorr = .75,
		undebuffcolorg = 0,
		undebuffcolorb = 0,
		warndebuffcolorr = .75,
		warndebuffcolorg = .75,
		warndebuffcolorb = 0,
		showmissingtalents = false,
		showbest = true,
		showclass = false,
		showhealingtaken = false,
		showcastspeed = false,
	},
}


--[[ ---------------------------------------------------------------------------
	 Toggle the config window
----------------------------------------------------------------------------- ]]
function DebuffEnough:ToggleConfigDialog()

   local frame = C.OpenFrames["DebuffEnough"]

   if frame then
	  C:Close("DebuffEnough")
   else
	  C:Open("DebuffEnough")
	  self:SetStatusText(format(L["Profile: %s"], self.db:GetCurrentProfile()))
   end

end


--[[ ---------------------------------------------------------------------------
	 Set the text displayed at the bottom of the config window
----------------------------------------------------------------------------- ]]
function DebuffEnough:SetStatusText(text)

   local frame = C.OpenFrames["DebuffEnough"]

   if frame then
	  frame:SetStatusText(text)
   end

end


--[[ ---------------------------------------------------------------------------
	 Update config display on a profile change
----------------------------------------------------------------------------- ]]
function DebuffEnough:OnProfileChanged(event, newdb)

   if event ~= "OnProfileDeleted" then
	  if LibStub("LibLogger-1.0", true) then self:SetLogLevel(self.db.profile.logLevel) end
	  self:SetStatusText(format(L["Profile: %s"], self.db:GetCurrentProfile()))
   end

end


--[[ ---------------------------------------------------------------------------
	 Set a saved variable config option
----------------------------------------------------------------------------- ]]
function DebuffEnough:SetProfileParam(var, value)

   local varName = nil

   if type(var) == "string" then
	   varName = var
   else
	   varName = var[#var]
   end

   if self.trace then self:trace("Setting parameter %s to %s.", varName, tostring(value)) end

   self.db.profile[varName] = value

   if varName == "logLevel" and LibStub("LibLogger-1.0", true) then
	  self:SetLogLevel(value)
   end

end


--[[ ---------------------------------------------------------------------------
	 Get a saved variable config option
----------------------------------------------------------------------------- ]]
function DebuffEnough:GetProfileParam(var) 

   local varName = nil

   if type(var) == "string" then
	   varName = var
   else
	   varName = var[#var]
   end

   if self.spam then self:spam("Getting parameter %s as %s", varName, tostring(self.db.profile[varName])) end

   return self.db.profile[varName]

end